<?php
class BifitKassaTools
{
	const MODULE_ID = "bifit.kassa";

	public function __construct()
	{
	}

	public static function __clearString($str)
	{
		$arr = ["\\", "/", "\n", "\r", "\t", "\x08", "\x0c"];
		$str = str_replace($arr, " ", $str);
		$str = str_replace("Ð", "D", $str);
		$str = preg_replace(
			"/\s\s+/",
			" ",
			preg_replace(
				"/[^ а-яА-ЯёЁ0-9a-zA-Z?@№#()!,-.:;]/ui",
				"",
				strip_tags(str_replace(" ", " ", $str))
			)
		);

		return trim($str);
	}

	public static function __createGUID($order)
	{
		// Create GUID (Globally Unique Identifier)
		$guid = "";
		$namespace = rand(11111, 99999);
		$uid = uniqid("", true);
		$data = $namespace;
		$data .= $_SERVER["REQUEST_TIME"];
		$data .= $_SERVER["HTTP_USER_AGENT"];
		$data .= $_SERVER["REMOTE_ADDR"];
		$data .= $_SERVER["REMOTE_PORT"];
		$data .= $order["number"];
		$data .= $order["summ"];
		$hash = strtoupper(hash("ripemd128", $uid . $guid . md5($data)));
		$guid =
			substr($hash, 0, 8) .
			"-" .
			substr($hash, 8, 4) .
			"-" .
			substr($hash, 12, 4) .
			"-" .
			substr($hash, 16, 4) .
			"-" .
			substr($hash, 20, 12);

		return strtolower($order["cms"] . "-" . $guid);
	}

	public static function __saveLogfile($data, $order_number)
	{
		include dirname(__FILE__) . "/../../install/version.php";
		$version =
			is_array($arModuleVersion) &&
			array_key_exists("VERSION", $arModuleVersion)
				? $arModuleVersion["VERSION"]
				: "???";

		$file =
			dirname(__FILE__) .
			"/logs/log_" .
			str_replace(
				['"', "'", "\/", "/", ":", "*", "?", "<", ">", "|"],
				"-",
				$order_number
			) .
			".txt";
		$top =
			"## " .
			date("Y-m-d H:i:s") .
			" | " .
			GetMessage("BIFIT_LOG_BIFIT_KASSA") .
			" (" .
			$version .
			") | " .
			GetMessage("BIFIT_LOG_ORDER_NUMBER") .
			$order_number;
		$len = mb_strlen($top, "UTF-8");

		for ($i = 0; $i < $len; $i++) {
			$line .= "#";
		}

		$current = file_get_contents($file);
		$content =
			$current . $line . "\n" . $top . "\n" . $line . "\n" . $data . "\n";
		file_put_contents($file, $content);
	}

	protected static function __getOrderPayments(
		$orderId,
		$pmBonusIds = [],
		$paid = null
	) {
		$payments = [];
		$dbRes = ("\Bitrix\Sale\PaymentCollection")::getList([
			"select" => ["*"],
			"order" => ["DATE_PAID" => "ASC"],
			"filter" => [
				"ORDER_ID" => $orderId,
				"PAID" => "Y"
			]
		]);

		while ($item = $dbRes->fetch()) {
			if (
				is_array($pmBonusIds) &&
				count($pmBonusIds) &&
				in_array($item["PAY_SYSTEM_ID"], $pmBonusIds)
			) {
				continue;
			}

			if ($paid && $item["PAID"] != "Y") {
				continue;
			}

			$payments[] = $item;
		}

		return $payments;
	}

	static function __getOrderTypes($order_id)
	{
		global $DB;

		$order_id = intval($order_id);

		$strSql =
			"SELECT *	
				 FROM `b_bifit_kassa_reports` 
				WHERE `ORDER_ID` = " . $order_id;
		$db_res = $DB->Query(
			$strSql,
			false,
			"File: " . __FILE__ . "<br>Line: " . __LINE__
		);

		$arr_types = [];

		while ($res = $db_res->Fetch()) {
			if ((int) $res["RECEIPT_ID"] > 0) {
				$arr_types[] = $res["RECEIPT_TYPE"];
			}
		}

		$arr_types = count($arr_types) ? array_unique($arr_types) : $arr_types;

		return $arr_types;
	}

	static function __getPayment($orderId, $pmBonusIds = [])
	{
		$payments = self::__getOrderPayments($orderId);
		$summ = 0;

		foreach ($payments as $item) {
			if (
				is_array($pmBonusIds) &&
				count($pmBonusIds) &&
				!in_array($item["PAY_SYSTEM_ID"], $pmBonusIds)
			) {
				continue;
			}

			$summ += $item["SUM"];
		}

		return $summ;
	}

	static function __formatprc($price)
	{
		CModule::IncludeModule("sale");

		$decimals = 2;
		$formatInfo = CCurrencyLang::GetCurrencyFormat(
			COption::GetOptionString("sale", "default_currency", "RUB")
		);

		if (isset($formatInfo["DECIMALS"])) {
			$decimals = intval($formatInfo["DECIMALS"]);
		}

		$dec_count = $decimals;

		$price =
			$dec_count < 0
				? round($price, $dec_count)
				: number_format($price, $dec_count, ".", "");

		return $price;
	}

	static function __checkInn($inn, $type = 1)
	{
		// check the fullness
		if (isset($inn) && trim($inn) != "") {
			if ($type == 2) {
				// only numbers in the amount of 10 pcs.
				$inn = !preg_match("/^\d{10}$/", trim($inn))
					? GetMessage("BIFIT_WRONG_INN")
					: trim($inn);
			} elseif ($type == 1) {
				// only numbers in the amount of 12 pcs.
				$inn = !preg_match("/^\d{12}$/", trim($inn))
					? GetMessage("BIFIT_WRONG_INN")
					: trim($inn);
			} else {
				// only numbers in the amount of 10 or 12 pieces.
				$inn = !preg_match("/^\d{10}$|^\d{12}$/", trim($inn))
					? GetMessage("BIFIT_WRONG_INN")
					: trim($inn);
			}
		} else {
			// TIN is empty or not found!
			$inn = GetMessage("BIFIT_EMPTY_INN");
		}

		return $inn;
	}

	static function __checkPhone($tel)
	{
		// check the fullness
		if (isset($tel) && trim($tel) != "") {
			// remove extra characters
			$phone = str_replace(["+", "(", ")", "-", " "], "", trim($tel));

			// get the first digit
			$first = substr($phone, "0", 1);

			// if the number is 10 digits and the first digit is not 7, then add 7
			$phone =
				strlen($phone) == 10 && $first != 7 ? "7" . $phone : $phone;

			// Final inspection
			$phone = !preg_match("/^[0-9]{11}+$/", $phone)
				? str_replace(
					GetMessage("BIFIT_PHONE"),
					GetMessage("BIFIT_PHONE") . " " . $phone,
					GetMessage("BIFIT_WRONG_PHONE")
				)
				: $phone;
		} else {
			// The phone is empty or not found!
			$phone = GetMessage("BIFIT_EMPTY_PHONE");
		}

		return $phone;
	}

	static function __checkEmail($email)
	{
		// check fullness
		if (isset($email) && trim($email) != "") {
			// record model check
			$email = !preg_match("/^\+\d+$|^\S+@\S+$/", $email)
				? str_replace(
					GetMessage("BIFIT_EMAIL"),
					GetMessage("BIFIT_EMAIL") . " " . $email,
					GetMessage("BIFIT_WRONG_EMAIL")
				)
				: $email;
		} else {
			// Email is empty or not found!
			$email = GetMessage("BIFIT_EMPTY_EMAIL");
		}

		return $email;
	}

	static function __checkPrice(
		$item_price,
		$order_subtotal,
		$order_discount = 0
	) {
		// if the discount is more than zero
		if ($order_discount > 0) {
			// calculate the discount percentage
			$percent = ($order_discount * 100) / $order_subtotal;

			// reduce the price of goods according to the discount
			$item_price = $item_price - $item_price * ($percent / 100);
		}

		return $item_price;
	}

	static function __checkResponce($response)
	{
		$mess = "";

		if (
			is_object($response) &&
			($response->type == "ERROR" ||
				$response->status == 400 ||
				$response->status == 500)
		) {
			$mess = "";

			if ($response->status == 400 || $response->status == 500) {
				$mess .= $response->error . "\n";
				$mess .= $response->message . "\n";
			} else {
				$array = json_decode(json_encode($response), true);
				array_walk_recursive($array, function ($item, $key) use (
					&$mess
				) {
					if ($item != "ERROR" && trim($item != "")) {
						$mess .= $key . ": " . $item . "\n";
					}
				});
			}
		} elseif (is_object($response)) {
			$array = json_decode(json_encode($response), true);
			array_walk_recursive($array, function ($item, $key) use (&$mess) {
				if (trim($item != "")) {
					$mess .= $key . ": " . $item . "\n";
				}
			});
		}

		return $mess;
	}

	static function __loadData(
		$url,
		$headers = false,
		$data = false,
		$type = "POST"
	) {
		// initialize the cURL session
		$ch = curl_init($url);

		// assign request type
		if ($type != "POST") {
			curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $type);
		} else {
			curl_setopt($ch, CURLOPT_POST, true);
		}

		// add data
		if ($data) {
			curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
		}

		// returning the result as a string
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_ENCODING, "");
		curl_setopt($ch, CURLOPT_MAXREDIRS, 10);
		curl_setopt($ch, CURLOPT_TIMEOUT, 0);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
		curl_setopt($ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);

		// add headers
		if ($headers) {
			curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
		}

		// get the answer
		$output = curl_exec($ch);

		// close the cURL session
		curl_close($ch);

		return $output;
	}

	static function __getToken($login, $password)
	{
		$signature = urlencode(base64_encode(hash("sha256", $password, true)));
		$data = "username=" . $login;
		$data .= "&password=" . $signature;
		$data .= "&client_id=cashdesk-rest-client";
		$data .= "&client_secret=cashdesk-rest-client";
		$data .= "&grant_type=password";
		$url = "https://kassa.bifit.com/cashdesk-api/v1/oauth/token";
		$headers = ["Content-Type: application/x-www-form-urlencoded"];
		$tokenData = json_decode(self::__loadData($url, $headers, $data));

		return $tokenData;
	}

	static function __getOrganizations($token)
	{
		$url =
			"https://kassa.bifit.com/cashdesk-api/v1/protected/organizations/list/read";
		$headers = [
			"Authorization: Bearer " . $token,
			"Content-Type: application/json",
			"Content-Length: 0"
		];
		$orgData = json_decode(self::__loadData($url, $headers));

		return $orgData;
	}

	static function __getTradeObjects($org_id, $token)
	{
		$data = "organization_id=" . $org_id;
		$url =
			"https://kassa.bifit.com/cashdesk-api/v1/protected/trade_objects/list/read";
		$headers = [
			"Content-Type: application/x-www-form-urlencoded",
			"Authorization: Bearer " . $token
		];
		$tradeData = json_decode(self::__loadData($url, $headers, $data));

		return $tradeData;
	}

	static function __createMainArray($orgData, $token)
	{
		$arr = [];

		foreach ($orgData as $key => $org) {
			$arr[$key] = new stdClass();
			$arr[$key]->id = $org->id;
			$arr[$key]->name = $org->name;
			$arr[$key]->traides = [];

			$tradeData = self::__getTradeObjects($org->id, $token);

			foreach ($tradeData as $k => $trd) {
				$arr[$key]->traides[$k] = new stdClass();
				$arr[$key]->traides[$k]->id = $trd->id;
				$arr[$key]->traides[$k]->name = $trd->name;
			}
		}

		return $arr;
	}

	static function __createSelects($array, $name, $org_id = null)
	{
		$html = '<select name="' . $name . '">';

		foreach ($array as $key => $org) {
			if ($name == "ORGANIZATION") {
				$html .= '<option value="' . $org->id . '">';
				$html .= $org->name;
				$html .= "</option>";
			}

			if ($name == "TRADE_OBJECT" && $org->id != $org_id) {
				continue;
			}

			if ($name != "ORGANIZATION") {
				foreach ($org->traides as $k => $trd) {
					$html .= '<option value="' . $trd->id . '">';
					$html .= $trd->name;
					$html .= "</option>";
				}
			}
		}

		$html .= "</select>";

		return $html;
	}

	static function __saveReport($order, $receipt_id, $doc, $bifit)
	{
		// display the property of the operation
		$log =
			$order->test != ""
				? "####### " . GetMessage("BIFIT_TEST_REQUEST") . "\n"
				: "####### " . GetMessage("BIFIT_BATTLE_REQUEST") . "\n";

		// set the time zone according to the site settings
		$log .= "####### " . date("Y-m-d H:i:s", time()) . "\n";

		// ANSWER #1

		$log .= "####### " . GetMessage("BIFIT_ANSWER") . " #1: " . "\n";

		// response data # 1
		$token_error = null;

		if (is_object($bifit)) {
			foreach ($bifit as $key => $val) {
				if ($key == "error") {
					$token_error = 1;
				}

				$log .= $key . ": " . $val . "\n";
			}
		}

		// RECEIPT

		// generated receipt for sending
		$log .=
			"####### " .
			GetMessage("BIFIT_CHECK") .
			": " .
			"\n" .
			$order->receipt .
			"\n";

		if (!$token_error) {
			// IDEMPOTENCY KEY

			$log .=
				"####### " .
				GetMessage("BIFIT_IDEMPOTENCY_KEY") .
				": " .
				"\n" .
				$order->idemp_key .
				"\n";

			// ANSWER #2

			$log .= "####### " . GetMessage("BIFIT_ANSWER") . " #2: " . "\n";

			// response data #2
			if (is_object($receipt_id)) {
				$log .= self::__checkResponce($receipt_id);
				$order->receipt_id = GetMessage("BIFIT_NONE_SM");
			} else {
				$order->receipt_id = $receipt_id;
				$log .=
					GetMessage("BIFIT_DOCUMENT_NUMBER") .
					": " .
					$receipt_id .
					"\n";

				// ANSWER #3

				$log .=
					"####### " . GetMessage("BIFIT_ANSWER") . " #3: " . "\n";

				if (is_object($doc)) {
					$log .= self::__checkResponce($doc);
				}

				// LINK TO THE CHECK

				// generated link to the check
				$log .=
					"####### " .
					GetMessage("BIFIT_LINK_TO_CHECK") .
					": " .
					"\n" .
					$order->receipt_link;
			}
		}

		$order->log = $log;

		// SAVE IN DB

		global $DB;

		$connection = ("\Bitrix\Main\Application")::getConnection();
		$sqlHelper = $connection->getSqlHelper();
		$connection = ("\Bitrix\Main\Application")::getConnection();
		$connection->queryExecute(
			"INSERT INTO b_bifit_kassa_reports (
			ORDER_ID, 
			ORDER_NUMBER,
			ORDER_DATE,
			ACTION_DATE,
			RECEIPT_LINK,
			RECEIPT_ID,
			RECEIPT_TYPE,
			LOG
			) VALUES ('" .
				$sqlHelper->forSql($order->id) .
				"', 
			'" .
				$sqlHelper->forSql($order->number) .
				"', 
			'" .
				$sqlHelper->forSql($order->created) .
				"', 
			'" .
				$sqlHelper->forSql($order->changed) .
				"', 
			'" .
				$sqlHelper->forSql($order->receipt_link) .
				"', 
			'" .
				$sqlHelper->forSql($order->receipt_id) .
				"', 
			'" .
				$sqlHelper->forSql($order->receipt_type) .
				"',
			'" .
				$sqlHelper->forSql($order->log) .
				"'
			)"
		);
	}

	static function __getOrderTransIds($order_id, $org_id, $token)
	{
		// link
		$url =
			"https://kassa.bifit.com/cashdesk-api/v1/protected/online_orders/details/" .
			$order_id .
			"?organization_id=" .
			$org_id;

		// headers
		$headers = [
			"Content-Type: application/x-www-form-urlencoded",
			"Authorization: Bearer " . $token
		];

		// data
		$type = "GET";
		$data = false;

		// result
		$orderContainer = json_decode(
			self::__loadData($url, $headers, $data, $type)
		);
		$ids = [];

		if (is_object($orderContainer)) {
			$onlineOrderPayments = $orderContainer->onlineOrderPayments;

			if (count($onlineOrderPayments)) {
				foreach ($onlineOrderPayments as $onlinePayment) {
					$ids[$onlinePayment->id] = $onlinePayment->transactionId;
				}
			}
		}

		return $ids;
	}

	public static function __getOneThingKey($products, $diff)
	{
		$one_thing_key = null;

		foreach ($products as $key => $product) {
			if ($product->count == 1 && $product->price > abs($diff)) {
				$one_thing_key = $key;
				break;
			}
		}

		return $one_thing_key;
	}

	public static function __getOrderData($arFields)
	{
		// order
		$order = CSaleOrder::GetByID($arFields["ID"]);

		// params
		$mid = self::MODULE_ID;
		$user_phone = null;
		$user_email = null;
		$codes = ["LOCATION", "ZIP", "CITY", "ADDRESS"];
		$uaa = [];
		$op = CSaleOrderPropsValue::GetList(
			["SORT" => "ASC"],
			["ORDER_ID" => $arFields["ID"]]
		);

		while ($aop = $op->Fetch()) {
			$opsid = $aop["ORDER_PROPS_ID"];
			$value = $arFields["ORDER_PROP"][$opsid];

			if ($aop["CODE"] == "EMAIL" && $aop["VALUE"] != "") {
				$user_email =
					isset($value) && $value != "" && $value != $aop["VALUE"]
						? $value
						: $aop["VALUE"];
			}
			if ($aop["CODE"] == "PHONE" && $aop["VALUE"] != "") {
				$user_phone =
					isset($value) && $value != "" && $value != $aop["VALUE"]
						? $value
						: $aop["VALUE"];
			}
			if (in_array($aop["CODE"], $codes) && $aop["VALUE"] != "") {
				if ($aop["CODE"] == "LOCATION") {
					$city_id =
						isset($value) && $value != "" && $value != $aop["VALUE"]
							? $value
							: $aop["VALUE"];
					$res = ("\Bitrix\Sale\Location\LocationTable")::getList([
						"filter" => [
							"=CITY_ID" => $city_id,
							"=PARENTS.NAME.LANGUAGE_ID" => LANGUAGE_ID,
							"=PARENTS.TYPE.NAME.LANGUAGE_ID" => LANGUAGE_ID
						],
						"select" => [
							"I_ID" => "PARENTS.ID",
							"I_NAME_RU" => "PARENTS.NAME.NAME",
							"I_TYPE_CODE" => "PARENTS.TYPE.CODE",
							"I_TYPE_NAME_RU" => "PARENTS.TYPE.NAME.NAME"
						],
						"order" => [
							"PARENTS.DEPTH_LEVEL" => "asc"
						]
					]);
					while ($item = $res->fetch()) {
						if (!in_array($item["I_NAME_RU"], $uaa)) {
							$uaa[] = $item["I_NAME_RU"];
						}
					}
				} else {
					$uaa[] =
						isset($value) && $value != "" && $value != $aop["VALUE"]
							? $value
							: $aop["VALUE"];
				}
			}
		}

		$user_address = count($uaa) ? implode(", ", $uaa) . "." : null;
		$address_type = COption::GetOptionString($mid, "address_type", "email");

		// check bonus discounts
		$bonus_summ = 0;

		if (COption::GetOptionString($mid, "bonus")) {
			$pm_bonus_ids = json_decode(
				COption::GetOptionString($mid, "pm_bonus_ids")
			);
			$bonus_summ = self::__getPayment($arFields["ID"], $pm_bonus_ids);
		}

		// discounts summ
		$order_discount = $bonus_summ + abs($arFields["DISCOUNT_VALUE"]);

		// products
		$products = [];
		$basket_items = $arFields["BASKET_ITEMS"];

		// basket
		foreach ($basket_items as $k => $product) {
			$tax_value =
				$product["VAT_INCLUDED"] == "Y" ? 0 : $arFields["TAX_VALUE"];
			$price = self::__formatprc(
				self::__checkPrice(
					$product["PRICE"],
					$arFields["PRICE"] + $tax_value,
					$order_discount
				)
			);

			$articul = $product["PROPS"]["ARTNUMBER"]["VALUE"];

			$products[$k] = new stdClass();
			$products[$k]->id = $product["PRODUCT_ID"];
			$products[$k]->paymentsubject = "PRODUCT";
			$products[$k]->name = self::__clearString($product["NAME"]);
			$products[$k]->base_price = $product["PRICE"];
			$products[$k]->price = $price;
			$products[$k]->count = (float) $product["QUANTITY"];
			$products[$k]->tax =
				(float) $product["VAT_RATE"] > 0
					? "VAT_" . (int) ($product["VAT_RATE"] * 100)
					: "WITHOUT_VAT";
			$products[$k]->tax_value =
				(float) $product["VAT_RATE"] > 0
					? (int) ($product["VAT_RATE"] * 100)
					: 0;
			$products[$k]->code = $articul;
			$products[$k]->correction = 0;

			$cart_new_summ += self::__formatprc($price * $product["QUANTITY"]);
		}

		// correct delivery price
		$order_delivery_price = 0;

		if ($arFields["PRICE_DELIVERY"] > 0) {
			$order_delivery_price = self::__formatprc(
				self::__checkPrice(
					$arFields["PRICE_DELIVERY"],
					$arFields["PRICE"],
					$order_discount
				)
			);
		}

		// correct total summ
		$correction = null;
		$new_order_price = $cart_new_summ + $order_delivery_price;
		$order_price = $arFields["PRICE"] - $bonus_summ;

		if (
			$order_price > 0 &&
			$new_order_price > 0 &&
			$order_price != $new_order_price
		) {
			$diff = self::__formatprc($order_price - $new_order_price);

			// Correction from 03/03/2023

			$one_thing_key = self::__getOneThingKey($products, $diff);

			if ($one_thing_key) {
				$products[$one_thing_key]->price_before =
					$products[$one_thing_key]->price;
				$products[$one_thing_key]->price += round($diff, 2);
				$products[$one_thing_key]->correction = 1;
				$correction = 1;
			} else {
				// /correction from 03/03/2023

				foreach ($products as $key => $product) {
					if ($product->price > 0 && $product->price > abs($diff)) {
						if (
							$product->count == 1 ||
							$product->count == abs($diff) * 100 ||
							($product->count % 2 == 0 &&
								(abs($diff) * 100) % 2 == 0) ||
							($product->count % 2 == 1 &&
								(abs($diff) * 100) % 2 == 1)
						) {
							$res = abs($diff) / $product->count;
							$exp = explode(".", $res);
							$zer = isset($exp[1]) ? strlen($exp[1]) : 0;

							if ($res >= 0.01 && $zer <= 2) {
								$products[$key]->price_before = $product->price;
								$products[$key]->price += self::__formatprc(
									$diff / $product->count
								);

								// NEW ADDITIONAL CHECK AND FIX

								$summ_1 =
									self::__formatprc(
										$products[$key]->price_before *
											$product->count
									) + $diff;
								$summ_2 = self::__formatprc(
									$products[$key]->price * $product->count
								);

								if ($summ_1 != $summ_2) {
									$products[$key]->price = self::__formatprc(
										$summ_1 / $product->count
									);
								}

								// /new additional check and fix

								$products[$key]->correction = 1;
								$correction = 1;
								break;
							}
						}
					}
				}

				// Correction from 03/03/2023
			}

			// /correction from 03/03/2023
		}

		$arFields["PRICE"] = $correction ? $order_price : $new_order_price;

		// final correction
		if ($order_price != $arFields["PRICE"]) {
			$new_key = count($products);

			foreach ($products as $key => $product) {
				if ($product->count > 1) {
					$products[$new_key] = clone $product;
					$products[$new_key]->count--;

					$products[$key]->price_before = $product->price;
					$products[$key]->price += $diff;
					$products[$key]->count = 1;
					$products[$key]->correction = 1;

					$arFields["PRICE"] = $order_price;
					break;
				}
			}
		}

		if ($order_price <= 0) {
			foreach ($products as $key => $product) {
				$products[$key]->price = 0;
			}

			$arFields["PRICE_DELIVERY"] = 0;
			$arFields["PRICE"] = 0;
		}

		// Avans
		$avans = COption::GetOptionString($mid, "use_type_0") ? 1 : null;
		$summ_paid =
			$arFields["SUM_PAID"] > 0
				? self::__formatprc($arFields["SUM_PAID"] - $bonus_summ)
				: self::__formatprc($arFields["PRICE"] - $bonus_summ);

		if ($avans) {
			$product = new stdClass();
			$product->id = "avans" . $summ_paid;
			$product->paymentsubject = "PAYMENT";
			$product->name = "АВАНС";
			$product->price = $summ_paid;
			$product->count = 1;
			$product->tax = "WITHOUT_VAT";
			$product->tax_value = 0;
			$product->code = "avans" . $summ_paid;
		}

		// delivery
		$delivery = CSaleDelivery::GetByID($arFields["DELIVERY_ID"]);
		$vat_id = $delivery["VAT_ID"];
		$vatList = [];

		if (("\Bitrix\Main\Loader")::includeModule("catalog")) {
			$dbRes = ("\Bitrix\Catalog\VatTable")::getList([
				"filter" => ["ACTIVE" => "Y"],
				"order" => ["SORT" => "ASC"]
			]);

			while ($vat = $dbRes->fetch()) {
				$vatList[$vat["ID"]] = (int) $vat["RATE"];
			}
		}

		$delivery_tax =
			$vatList[$vat_id] > 0 ? "VAT_" . $vatList[$vat_id] : "WITHOUT_VAT";
		$delivery_tax_value = $vatList[$vat_id] > 0 ? $vatList[$vat_id] : 0;

		// data
		$data = new stdClass();
		$data->id = $arFields["ID"];
		$data->number = $arFields["ACCOUNT_NUMBER"];
		$data->created = $order["DATE_INSERT"];
		$data->changed = date("Y-m-d H:i:s");
		$data->total = self::__formatprc($arFields["PRICE"]);
		$data->delivery_id = $arFields["DELIVERY_ID"];
		$data->delivery_cost = $order_delivery_price;
		$data->delivery_tax = $delivery_tax;
		$data->delivery_tax_value = $delivery_tax_value;
		$data->payment_id = $arFields["PAY_SYSTEM_ID"];
		$data->status_id = $arFields["STATUS_ID"];
		$data->paid = $arFields["PAYED"];
		$data->pay_date = $arFields["DATE_PAYED"];
		$data->sum_paid = self::__formatprc(
			$arFields["SUM_PAID"] - $bonus_summ
		);
		$data->payments = self::__getOrderPayments($arFields["ID"]);
		$data->comment = self::__clearString($arFields["COMMENTS"]);
		$data->user_comment = self::__clearString(
			$arFields["USER_DESCRIPTION"]
		);
		$data->user_name = self::__clearString($order["USER_NAME"]);
		$data->user_last_name = self::__clearString($order["USER_LAST_NAME"]);
		$data->user_email =
			isset($arFields["USER_EMAIL"]) && $arFields["USER_EMAIL"] != ""
				? self::__checkEmail($arFields["USER_EMAIL"])
				: $user_email;
		$data->user_phone =
			isset($user_phone) && $user_phone != ""
				? self::__checkPhone($user_phone)
				: null;
		$data->user_address =
			isset($user_address) && $user_address != ""
				? self::__clearString($user_address)
				: null;
		$data->address =
			$address_type == "email" ? $data->user_email : $data->user_phone;
		$data->products = $products;
		$data->cashier_name = COption::GetOptionString($mid, "cashier_name");
		$data->cashier_inn = COption::GetOptionString($mid, "cashier_inn");
		$data->payment_address = COption::GetOptionString(
			$mid,
			"payment_address"
		);
		$data->calc_method = COption::GetOptionString(
			$mid,
			"calculation_method"
		);
		$data->payment_type = COption::GetOptionString($mid, "payment_type");
		$data->fio = COption::GetOptionString($mid, "fio");
		$data->vat_mode = COption::GetOptionString($mid, "vat_mode");
		$data->vat = COption::GetOptionString($mid, "vat");
		$data->vat_value = str_replace(
			["WITHOUT_VAT", "VAT_"],
			["0", ""],
			strtoupper($data->vat)
		);

		// Avans
		$data->avans = COption::GetOptionString($mid, "use_type_0") ? 1 : null;
		$data->avans_calc_method = COption::GetOptionString(
			$mid,
			"avans_calc_method"
		);
		$data->avans_payment_type = "CARD";
		$data->avans_products = [$product];

		// Bonus
		$data->bonus = COption::GetOptionString($mid, "bonus");
		$data->pm_bonus_ids = json_decode(
			COption::GetOptionString($mid, "pm_bonus_ids")
		);
		$data->bonus_summ = self::__getPayment($data->id, $data->pm_bonus_ids);

		// Payments
		$data->ok_payments = self::__getOrderPayments(
			$data->id,
			$data->pm_bonus_ids,
			1
		);

		return $data;
	}
}
